///////////////////////////////////////////////////////////////////////////////
//                                                         
// My2CegoAction.cc  
// -------------                                                        
// My2Cego semantic parser actions
//
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: My2CegoAction
// 
// Description: 
//
// Status:
//
///////////////////////////////////////////////////////////////////////////////


#include <ctype.h>
#include <stdlib.h>

#include "My2CegoAction.h"

#define MYSQL_MAXLINELEN 10000000

My2CegoAction::My2CegoAction(const Chain& fileName, CegoNet *pCegoNet, bool isCaseSensitive)
{
    _pFile = new File(fileName);   
    _pFile->open(File::READ);
    _i = 0;
    _pC = 0;
    _pCegoNet = pCegoNet;
    _isFirst=true;
    _isCaseSensitive = isCaseSensitive;
}

My2CegoAction::~My2CegoAction()  
{
}

bool My2CegoAction::moreData()
{
    while ( 1 )
    {
	if ( _pFile->readLine(_line, MYSQL_MAXLINELEN) > 0 )
	{	    
	    if ( _line.length() >= 2 )
	    {		    
		if (_line.cutTrailing(" \t").subChain(1, 2) != Chain("--"))
		{
		    _pC = (char*)_line;
		    _i=0;	       
		    return true;
		}
	    }
	}
	else
	{
	    return false;
	}
    }
}

char My2CegoAction::nextChar() 
{   
    
    if ( _isFirst )
    {
	if ( moreData() == false )
	    return 0;
	_isFirst=false;
    }

    if ( *_pC == '\'')
    {
	_pC++;
	readChain();
	return 0;
    }

    if ( *_pC == '/' && *(_pC+1) == '*' )
    {
	_pC++;
	_pC++;

	bool eoc=false;
	while ( ! eoc )
	{
	    while ( *_pC != '*' )
	    {
		_pC++;
		if ( *_pC == 0 ) 
		{
		    if ( moreData() == false )
			return 0; 
		}
	    }
	    if ( *_pC == '*' && *(_pC+1) == '/' )
	    {
		eoc=true;
		_pC++;
	    }	   
	    _pC++;	    
	}	
    }

    if ( *_pC == 0 ) 
    {
	if ( moreData() )
	    return nextChar();
	else
	    return 0; 
    }
        
    char c = *_pC; 
    _pC++; 

    if ( _isCaseSensitive )
	return c;
    else
	return (char)tolower((int)c);
} 

void My2CegoAction::readChain()
{

    setReserved(STRINGVAL); 
    
    int i=0;    

    
    while ( *_pC != '\'') 
    { 
	// cout << "Storing: " << *_pC << endl; 
	
	if ( *_pC == '\\') 
	{ 
	    _pC++; 
	    
	    if ( *_pC == 'n') 
	    { 
		_stringBuf[i] = '\n'; 
	    } 
	    else if ( *_pC == 't') 
	    { 
		_stringBuf[i] = '\t'; 
	    } 
	    else if ( *_pC == '\'') 
	    { 
		_stringBuf[i] = '\''; 
	    }
	    else if ( *_pC == '\\') 
	    { 
		_stringBuf[i] = '\\'; 
	    } 
	    else 
	    { 
		Chain msg = Chain("Invalid escape character <") + Chain(*_pC) + Chain(">"); 
		throw Exception(EXLOC, msg);
	    } 
	} 
	else
	{ 
	    _stringBuf[i] = *_pC; 
	} 
	
	i++; 
	
	if (i == MAXSTRINGLEN) 
	{ 
	    Chain msg("Stringbuf exceeded"); 
	    throw Exception(EXLOC, msg);
	}
	
	_pC++;
	
	if ( *_pC == 0 )
	{
	    Chain msg("Unterminated string"); 
	    throw Exception(EXLOC, msg);
	}
    }
    
    _pC++; 

    _stringBufLen = i+1;
    _stringBuf[i] = 0; 

    return; 
}

void My2CegoAction::backChar() 
{
    if ( _pC > 0 )
	_pC--;
}
    
void My2CegoAction::printTokenList() 
{    
    cout << "TokenList is " << endl;
	
    ListT<Chain> tlist =  getTokenList();
    Chain* pS = tlist.First();
    while (pS)
    {
	cout << *pS << endl;
	pS = tlist.Next();
    }    
}

void My2CegoAction::cleanUp() 
{

}

void My2CegoAction::postAction()
{
    Chain *pStmt = _postList.First();
    while ( pStmt )
    {
	if ( _pCegoNet )
	    _pCegoNet->doQuery(*pStmt);
	else
	    cout << *pStmt << endl;
	pStmt = _postList.Next();
    }    
} 

void My2CegoAction::execCreate() 
{

    Chain stmt = Chain("create table ") + _table + Chain(" (");

    Chain *pCol = _colList.First();
    while ( pCol )
    {
	stmt += Chain(" ") + *pCol;
	pCol = _colList.Next();
	if ( pCol )
	    stmt += Chain(",");
	
    }
    stmt += Chain(");");
    _colList.Empty();   

    // cout << "Create = " << stmt << endl;
    if ( _pCegoNet )
	_pCegoNet->doQuery(stmt);
    else
	cout << stmt << endl;
    
}

void My2CegoAction::execInsert() 
{

    Chain stmt = Chain("insert into ") + _table + Chain(" values (");

    Chain *pVal = _valList.First();
    while ( pVal )
    {
	stmt += Chain(" ") + *pVal;
	pVal = _valList.Next();
	if ( pVal )
	    stmt += Chain(",");	
    }
    stmt += Chain(");");
    _valList.Empty();   

    if ( _pCegoNet )
	_pCegoNet->doQuery(stmt);
    else
	cout << stmt << endl;

}

void My2CegoAction::storeTable() 
{
    Chain* pS = getTokenList().First();
    _table = pS->cutTrailing("`");    
}

void My2CegoAction::execDrop()
{
    Chain stmt = Chain("drop if exists table ") + _table +  Chain(";");

    if ( _pCegoNet )
	_pCegoNet->doQuery(stmt); 
    else
	cout << stmt << endl;

}

void My2CegoAction::dropOpt1()
{    
}

void My2CegoAction::dropOpt2()
{
}

void My2CegoAction::storeAttrColumn()
{
    Chain* pS = getTokenList().First();
    
    Chain colString = pS->cutTrailing("`") + Chain(" ") + _dataTypeString;
    if ( _nullOptString.length() > 0 )
	colString += Chain(" ") + _nullOptString;

    if ( _defaultOptString.length() > 0 )
	colString += Chain(" ") + _defaultOptString;
    
    _colList.Insert(colString);
}

void My2CegoAction::storePrimaryKey()
{
    Chain stmt = Chain("create primary index on ") + _table + Chain("(");

    Chain *pKeyCol = _keyColList.First();
    while ( pKeyCol )
    {
	stmt += Chain(" ") + *pKeyCol;
	pKeyCol = _keyColList.Next();
	if ( pKeyCol )
	    stmt += Chain(",");	
    }
    stmt += Chain(");");

    _keyColList.Empty();

    _postList.Insert(stmt);
}

void My2CegoAction::storeKeyColumn()
{
    Chain* pS = getTokenList().First();
    _keyColList.Insert(pS->cutTrailing("`"));
}

void My2CegoAction::storeNullOpt()
{
}

void My2CegoAction::storeNotNullOpt()
{
}

void My2CegoAction::storeDefaultOpt()
{
}

void My2CegoAction::storeDefaultNullOpt()
{
}

void My2CegoAction::insertConstValue()
{
    _valList.Insert(_constValue);
}

void My2CegoAction::storeIntType()
{
    if ( _dim < 10 )
	_dataTypeString = Chain("int");
    else
	_dataTypeString = Chain("long");
}

void My2CegoAction::storeUnsignedType()
{
}

void My2CegoAction::storeVarcharType()
{

    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    
    _dataTypeString = Chain("string(") + Chain(*pS) + Chain(")");
}

void My2CegoAction::storeTimestampType()
{
    _dataTypeString = Chain("datetime");
}

void My2CegoAction::storeEnumType()
{
    // for now, we map enum to string 100
    _dataTypeString = Chain("string(100)");
}


void My2CegoAction::storeIntValue()
{
    Chain* pS = getTokenList().First();
    
    if ( pS->length() < 10 )
	_constValue = *pS;
    else
	_constValue = Chain("(long)") + *pS;
}

void My2CegoAction::storeNegIntValue()
{
    Chain* pS = getTokenList().First();
    
    if ( pS->length() < 10 )
	_constValue = Chain("-") + *pS;
    else
	_constValue = Chain("(long)-") + *pS;
}

void My2CegoAction::storeVarcharValue()
{
    _constValue = Chain("'") + Chain(_stringBuf) + Chain("'");
}

void My2CegoAction::storeNullValue()
{
    _constValue = Chain("null");
}

void My2CegoAction::storeDim()
{
    
    Chain* pS = getTokenList().First();
    pS = getTokenList().Next();
    _dim = pS->asInteger();
}


