#ifndef _MY2CEGOACTION_H_INCLUDED_
#define _MY2CEGOACTION_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// My2CegoAction.h  
// ---------------
// My2Cego semantic actions interface definition
//
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: My2CegoAction
// 
// Description:
//
// Status: XXX
//
///////////////////////////////////////////////////////////////////////////////

#include <lfcbase/StackT.h>
#include <lfcbase/ListT.h>
#include <lfcbase/File.h>

#include <cego/CegoNet.h>
#include "My2Cego.h"

#define MAXSTRINGLEN 100000

class My2CegoAction : public My2Cego {
    
public:
    
    My2CegoAction(const Chain& fileName, CegoNet *pCegoNet, bool isCaseSensitive = false);
    ~My2CegoAction();

    void cleanUp();
    bool moreData();
    void setFile(const Chain& fileName);
    char nextChar();
    void readChain();
    void backChar();

    void printTokenList();

    /////////////////////////////////////////////////
    /////////// start of semantic actions ///////////
    /////////////////////////////////////////////////

    void postAction();
    
    void execCreate();
    void execInsert();
    void setInsertTable();
    void execDrop();
    void dropOpt1();
    void dropOpt2();
    void storeAttrColumn();
    void storeTable();

    void storePrimaryKey();
    void storeKeyColumn();
    void storeNullOpt();
    void storeNotNullOpt();
    void storeDefaultOpt();
    void storeDefaultNullOpt();
    void insertConstValue();
    void storeIntType();
    void storeUnsignedType();
    void storeVarcharType();
    void storeTimestampType();
    void storeEnumType();
    void storeIntValue();
    void storeNegIntValue();
    void storeVarcharValue();
    void storeNullValue();
    void storeDim();

private:

    int _i;
    char *_pC;
    Chain _line;

    int _stringBufLen;
    char _stringBuf[MAXSTRINGLEN];

    File* _pFile;
    CegoNet* _pCegoNet;

    bool _isFirst;
    bool _isCaseSensitive;

    ListT<Chain> _colList;
    ListT<Chain> _keyColList;
    ListT<Chain> _valList;

    ListT<Chain> _postList;
    
    Chain _dataTypeString;
    Chain _nullOptString;
    Chain _defaultOptString; 
    Chain _table;
    Chain _constValue;
    int _dim;

};

#endif














